@@ -27,7 +27,7 @@ def _user_args_to_dict(user_list):
     return user_dict
 
 
-installed_targets = [target for target in interface.plugin_store.registry]
+installed_targets = list(interface.plugin_store.registry)
 if len(installed_targets) > 0:
     supported_targets_msg = "Support is currently installed for deployment to: {targets}".format(
         targets=", ".join(installed_targets)
@@ -161,7 +161,7 @@ def create_deployment(flavor, model_uri, target, name, config, endpoint):
     " about supported remote URIs for model artifacts, see"
     " https://mlflow.org/docs/latest/tracking.html"
     "#artifact-stores",
-)  # optional model_uri
+)
 @click.option(
     "--flavor",
     "-f",
@@ -288,8 +288,15 @@ def predictions_to_json(raw_predictions, output):
 
 
 @commands.command("predict")
-@optional_endpoint_param
-@optional_deployment_name
+@click.option(
+    "--name",
+    "name",
+    help="Name of the deployment. Exactly one of --name or --endpoint must be specified.",
+)
+@click.option(
+    "--endpoint",
+    help="Name of the endpoint. Exactly one of --name or --endpoint must be specified.",
+)
 @target_details
 @parse_input
 @parse_output
@@ -299,6 +306,9 @@ def predict(target, name, input_path, output_path, endpoint):
     """
     import pandas as pd
 
+    if (name, endpoint).count(None) != 1:
+        raise click.UsageError("Must specify exactly one of --name or --endpoint.")
+
     df = pd.read_json(input_path)
     client = interface.get_deploy_client(target)
 
@@ -307,16 +317,22 @@ def predict(target, name, input_path, output_path, endpoint):
         result = client.predict(name, df, endpoint=endpoint)
     else:
         result = client.predict(name, df)
-    if output_path:
-        with open(output_path, "w") as fp:
-            predictions_to_json(result, fp)
+    if output_path is not None:
+        result.to_json(output_path)
     else:
-        predictions_to_json(result, sys.stdout)
+        click.echo(result.to_json())
 
 
 @commands.command("explain")
-@optional_endpoint_param
-@optional_deployment_name
+@click.option(
+    "--name",
+    "name",
+    help="Name of the deployment. Exactly one of --name or --endpoint must be specified.",
+)
+@click.option(
+    "--endpoint",
+    help="Name of the endpoint. Exactly one of --name or --endpoint must be specified.",
+)
 @target_details
 @parse_input
 @parse_output
@@ -333,6 +349,9 @@ def explain(target, name, input_path, output_path, endpoint):
     """
     import pandas as pd
 
+    if (name, endpoint).count(None) != 1:
+        raise click.UsageError("Must specify exactly one of --name or --endpoint.")
+
     df = pd.read_json(input_path)
     client = interface.get_deploy_client(target)
 
